#!/usr/bin/bash

# this script fixes permissions in one or more test directories
# instead of test directory the test name is accepted, too
#   e.g. 'eptest' and 'ep' are the same
#
# what it actually does are two things:
#  1. make sure that files belong to correct group
#  2. make sure that group has rwX permissions to those files
#
# this script will be called by lvatest-fix-permissions via sudo

# source bash base library
# shellcheck disable=SC1091
source /usr/libexec/bash-base.bash || {
    echo "$0: fatal error: failed to source /usr/libexec/bash-base.bash" >&2
    exit 1
}

DATADIR=/home/data

function usage() {
    cat <<EOF
${0##*/}: usage: ${0##*/} <test-name> [..]
  fixes permissions and group ownership for one or more tests
  caller must be part of lvatest group and part of lvatest-<test-name> group
EOF
    bb_quit
}

# fixperms <path>
function fixperms() {
    (( $# == 2 )) || bb_fatal "fixperms was called with $# args instead of 2"

    local testdir="$1"
    local groupname="$2"

    [[ -d "$testdir" ]] || bb_fatal "fixperms: no such dir: $testdir"

    for f in config priv DELAY-EXCEPTIONS DELAY-ROLLOUT
    do
        if [ -e "$testdir/$f" ]
        then
            chgrp -R "$groupname" "$testdir/$f"
            chmod -R g+rwX,o-rwx "$testdir/$f"
        else
            echo "$0: warning: no such file/dir: '$testdir/$f'" >&2
        fi
    done
}

function checkperms() {
    local test_name group_name

    test_name="${1%test}"
    group_name="lvatest-${test_name}"

    for g in "${EXAMGROUPS[@]}"
    do
        if [[ "$g" == "$test_name" ]]
        then
            if [[ -d "$DATADIR/${g}test" ]]
            then
                fixperms "$DATADIR/${g}test" "$group_name"
            else
                echo "$0: no such directory: '$DATADIR/${g}test', skipping" >&2
            fi

            # nothing more for this test
            return 0
        fi
    done

    echo "$0: warning: you have no rights to access '$g', skipping" >&2
}

function main() {
    (( $# > 0 )) || usage

    [[ "${SUDO_USER+x}" == "x" ]] || \
        bb_fatal "it seems this script was not called via sudo, aborting"

    (( UID == 0 )) || bb_fatal "must be run as root, but you are UID=$UID"

    # resolv users group memberships
    USERGROUPS="$(groups "$SUDO_USER")" || \
        bb_fatal "failed to get groups for user '$SUDO_USER'"
    USERGROUPS="${USERGROUPS#*:}"
    declare -a EXAMGROUPS=()
    local g
    for g in $USERGROUPS
    do
        if [[ "$g" =~ ^lvatest-.*$ ]]
        then
            EXAMGROUPS+=("${g#lvatest-}")
        fi
    done

    while (( $# > 0 ))
    do
        checkperms "$1"
        shift
    done
}

main "$@"
bb_quit
